@@ -3,13 +3,14 @@
 from asgiref.sync import sync_to_async
 
 from discord.ext import commands, tasks
-from discord import Embed, utils, PermissionOverwrite, Color, NotFound
+from discord import Embed, utils, PermissionOverwrite, Color, NotFound, User
 
 from django.utils import timezone
 
 from datetime import timedelta
 
-from discordbot.models import AmongUsGame, AMONGUS_PLAYER_COLORS, AMONGUS_EMOJI_COLORS
+from discordbot.models import AmongUsGame, AMONGUS_PLAYER_COLORS, AMONGUS_EMOJI_COLORS, VierGewinntGame, VIERGEWINNT_NUMBER_EMOJIS
+from discordbot.botmodules.serverdata import DjangoConnection
 
 import requests
 import os
@@ -31,6 +32,11 @@ NO = '❌'
 
 DELETE = '❌'
 
+AMONGUS_TRACKER_INSTALL = """Konfiguriere die Tracker-Software mit folgenden Daten:
+                
+Der [Tracker](https://raw.githubusercontent.com/rafaelurben/django-discordbot/master/discordbot/files/amongus/tracker.py) funktioniert vorerst nur für Windows uns ist (noch) sehr buggy!
+Benötigt werden Python, Pip und alle [requirements](https://raw.githubusercontent.com/rafaelurben/django-discordbot/master/discordbot/files/amongus/requirements.txt)"""
+
 amongus_last_update = timezone.now()-timedelta(days=1)
 
 #####
@@ -62,7 +68,7 @@ class Games(commands.Cog):
         self.amongus_backgroundtasks.cancel()
 
 
-
+    # Fortnite
 
     @commands.command(
         brief="Erhalte Aktuelles zu Fortnite",
@@ -125,14 +131,13 @@ class Games(commands.Cog):
             raise commands.BadArgument(message="Scheinbar ist dieser Befehl nicht richtig konfiguriert.")
 
 
-
-
+    # Minecraft
 
     @commands.group(
         brief="Hauptcommand für alle Minecraft Befehle",
         description='Alle Minecraft Befehle beginnen mit diesem Befehl',
         aliases=['mc'],
-        help="Um eine Liste aller AmongUs-Befehle zu erhalten, gib den Command ohne Argumente ein.",
+        help="Um eine Liste aller AmongUs-Befehle zu erhalten, gib den Command ohne Unterbefehl ein.",
         usage="<Unterbefehl> <Argument>"
     )
     async def minecraft(self, ctx):
@@ -231,10 +236,10 @@ class Games(commands.Cog):
 
 
 
+    # AmongUs
+    ## AmongUs Tasks
 
-
-
-    @tasks.loop(seconds=1.0)
+    @tasks.loop(seconds=0.5)
     async def amongus_backgroundtasks(self):
         def log(*args):
             print('[AmongUs Background Tasks] -', *args)
@@ -260,7 +265,7 @@ class Games(commands.Cog):
                     textchannel = await category.create_text_channel(name=f"amongus-{ game.pk }", reason="Textkanal war nicht mehr vorhanden!", topic=f"AmongUs Spiel - ID: { game.pk }")
                 
                     game.text_channel_id = str(textchannel.id)
-                    game.save()
+                    await DjangoConnection._saveAmongUsGame(game)
 
                 try:
                     msg = await textchannel.fetch_message(int(game.text_message_id))
@@ -274,7 +279,7 @@ class Games(commands.Cog):
                     msg = await textchannel.send(embed=embed)
 
                     game.text_message_id = str(msg.id)
-                    game.save()
+                    await DjangoConnection._saveAmongUsGame(game)
 
                     for emoji in AMONGUS_EMOJI_COLORS:
                         await msg.add_reaction(emoji)
@@ -307,7 +312,8 @@ class Games(commands.Cog):
                                 m = vs.channel.guild.get_member(m_id)
                                 if m is None:
                                     m = await vs.channel.guild.fetch_member(m_id)
-                                await m.edit(mute=True)
+                                if not m.bot:
+                                    await m.edit(mute=True)
 
                     # Don't allow new players to speak
                     overwrites = {
@@ -355,8 +361,8 @@ class Games(commands.Cog):
                     fields.append((title, description))
 
                 embed = self.bot.getEmbed(
-                    title=f"AmongUs Game { id }",
-                    description=(f"""Infos über Gruppe Nr. { id }
+                    title=f"AmongUs Spiel (ID: { id })",
+                    description=(f"""Infos über dieses Spiel
 
                         Letzte Trackerdaten - { last_tracking_data }  
                         Letzte Aktualisierung - { last_edited }
@@ -370,20 +376,21 @@ class Games(commands.Cog):
                     """),
                     footertext=f"{DEAD} = Tot - {ALIVE} = Lebend",
                     color=0xFEDE29,
-                    inline=True,
+                    inline=False,
                     fields=fields,
                 )
                 await msg.edit(embed=embed)
             except Exception as e:
                 log('Error:', e)
 
-
     @amongus_backgroundtasks.before_loop
     async def amongus_backgroundtasks_before(self):
         print('[AmongUs Background Tasks] - Waiting for bot to be ready...')
         await self.bot.wait_until_ready()
         print('[AmongUs Background Tasks] - Started!')
 
+    ## AmongUs Commands
+
     @commands.group(
         brief="Hauptcommand für alle AmongUs Befehle",
         description='Alle AmongUs-Befehle beginnen mit diesem Befehl.',
@@ -402,16 +409,18 @@ class Games(commands.Cog):
                     ("/au create",      "Erstelle ein AmongUs Spiel"),
                     ("/au delete",      "Lösche dein AmongUs Spiel"),
                     ("/au reset",       "Setze dein AmongUs Spiel zurück"),
+                    ("/au config",      "Erhalte nochmals die Konfigurationsdaten für den Tracker"),
                 ]
             )
 
     @amongus.command(
         name="create",
-        aliases=['open', 'add'],
+        aliases=['open', 'add', '+'],
     )
+    @commands.guild_only()
     async def amongus_create(self, ctx):
-        if AmongUsGame.objects.filter(creator=ctx.database.user, guild=ctx.database.server).exists():
-            game = AmongUsGame.objects.get(creator=ctx.database.user, guild=ctx.database.server)
+        if await ctx.database.hasAmongUsGame():
+            game = await ctx.database.getAmongUsGame()
             textchannel = ctx.guild.get_channel(int(game.text_channel_id))
             voicechannel = ctx.guild.get_channel(int(game.voice_channel_id))
             if textchannel and voicechannel:
@@ -422,32 +431,31 @@ class Games(commands.Cog):
                 raise commands.BadArgument(message=f"Du hast bereits ein AmongUs-Spiel auf diesem Server, jedoch konnte der Textkanal nicht mehr gefunden werden! (ID { game.pk })")
         else:
             category = await getAmongUsCategory(ctx.guild)
-            textchannel = await category.create_text_channel(name="amongus-loading", reason="Benutzer hat AmongUs-Spiel erstellt.", topic="Dieser AmongUS Kanal wird gerade erstellt...")
-            voicechannel = await category.create_voice_channel(name="AmongUs LOADING", reason="Benutzer hat AmongUs-Spiel erstellt.")
-            game = AmongUsGame.objects.create(creator=ctx.database.user, guild=ctx.database.server, text_channel_id=str(textchannel.id), voice_channel_id=str(voicechannel.id))
-            await textchannel.edit(name=f"amongus-{ game.pk }", topic=f"AmongUs Spiel - { game.pk }")
-            await voicechannel.edit(name=f"AmongUs - { game.pk }")
+            textchannel = await category.create_text_channel(name=f"au-{ctx.author.name}#{ctx.author.discriminator}", reason="Benutzer hat AmongUs-Spiel erstellt.", topic=f"AmongUs Spiel von {ctx.author.name}#{ctx.author.discriminator}")
+            voicechannel = await category.create_voice_channel(name=f"[AU] - {ctx.author.name}#{ctx.author.discriminator}", reason="Benutzer hat AmongUs-Spiel erstellt.")
+            game = await ctx.database.createAmongUsGame(text_channel_id=str(textchannel.id), voice_channel_id=str(voicechannel.id))
 
             embed = ctx.getEmbed(
-                title=f"AmongUs Spiel erstellt ({ game.pk })", 
-                description="Konfiguriere die Tracker-Software mit folgender URL und folgendem API Key:", 
+                title=f"AmongUs Spiel erstellt", 
+                description=AMONGUS_TRACKER_INSTALL,
                 color=0xFEDE29,
                 fields=[
                     ("Url", str(game.get_tracker_url())),
+                    ("ID", str(game.pk)),
                     ("API-Key", str(game.api_key)),
                     ("Kanal", str(textchannel.mention))
                 ]
             )
             await ctx.author.send(embed=embed)
 
-
     @amongus.command(
         name="close",
-        aliases=['delete', 'del'],
+        aliases=['delete', 'del', '-'],
     )
+    @commands.guild_only()
     async def amongus_close(self, ctx):
-        if AmongUsGame.objects.filter(creator=ctx.database.user, guild=ctx.database.server).exists():
-            game = AmongUsGame.objects.get(creator=ctx.database.user, guild=ctx.database.server)
+        if await ctx.database.hasAmongUsGame():
+            game = await ctx.database.getAmongUsGame()
             textchannel = ctx.guild.get_channel(int(game.text_channel_id))
             voicechannel = ctx.guild.get_channel(int(game.voice_channel_id))
             if textchannel is not None:
@@ -456,7 +464,7 @@ class Games(commands.Cog):
                 await voicechannel.delete(reason="AmongUs Spiel wurde gelöscht!")
 
             id = str(game.pk)
-            game.delete()
+            await ctx.database._deleteAmongUsGame(game)
 
             embed = ctx.getEmbed(
                 title=f"AmongUs Spiel gelöscht! ({ id })",
@@ -471,13 +479,15 @@ class Games(commands.Cog):
         name="reset",
         aliases=[],
     )
+    @commands.guild_only()
     async def amongus_reset(self, ctx):
-        if AmongUsGame.objects.filter(creator=ctx.database.user, guild=ctx.database.server).exists():
-            game = AmongUsGame.objects.get(creator=ctx.database.user, guild=ctx.database.server)
+        if await ctx.database.hasAmongUsGame():
+            game = await ctx.database.getAmongUsGame()
             voicechannel = ctx.guild.get_channel(int(game.voice_channel_id))
 
             if voicechannel is not None:
-                game.reset(save=True)
+                game.reset()
+                await ctx.database._saveAmongUsGame(game)
                 await voicechannel.edit(sync_permissions=True)
                 await ctx.sendEmbed(
                     title="AmongUs Spiel zurückgesetzt!",
@@ -489,5 +499,76 @@ class Games(commands.Cog):
         else:
             raise commands.BadArgument(message="Du hast kein AmongUs-Spiel auf diesem Server!")
 
+    @amongus.command(
+        name="apikey",
+        aliases=['resend', 'config', 'tracker', 'key'],
+    )
+    @commands.guild_only()
+    async def amongus_apikey(self, ctx):
+        if await ctx.database.hasAmongUsGame():
+            game = await ctx.database.getAmongUsGame()
+
+            embed = ctx.getEmbed(
+                title=f"AmongUs Tracker Konfiguration",
+                description=AMONGUS_TRACKER_INSTALL,
+                color=0xFEDE29,
+                fields=[
+                    ("Url", str(game.get_tracker_url())),
+                    ("ID", str(game.pk)),
+                    ("API-Key", str(game.api_key)),
+                ]
+            )
+            await ctx.author.send(embed=embed)
+        else:
+            raise commands.CommandError("DU hast kein AmongUs Spiel!")
+
+
+    # VierGewinnt
+
+    @commands.group(
+        brief="Hauptcommand für alle VierGewinnt Befehle",
+        description='Alle VierGewinnt-Befehle beginnen mit diesem Befehl.',
+        aliases=['fourinarow', '4gewinnt', '4inarow', '4row', '4win'],
+        help="Um eine Liste aller VierGewinnt-Befehle zu erhalten, gib den Command ohne Argumente ein.",
+        usage="<Unterbefehl> [Argumente]"
+    )
+    @commands.guild_only()
+    async def viergewinnt(self, ctx):
+        if ctx.invoked_subcommand is None:
+            await ctx.sendEmbed(
+                title="VierGewinnt Befehle",
+                description=f"""Alle AmongUs Befehle können ebenfalls mit `/viergewinnt` verwendet werden.""",
+                color=0x0078D7,
+                inline=False,
+                fields=[
+                    ("/4gewinnt duell <Mitglied>", "Duelliere einen anderen Spieler"),
+                ]
+            )
+
+    @viergewinnt.command(
+        name="duell",
+        aliases=['battle', 'duel'],
+    )
+    @commands.guild_only()
+    async def viergewinnt_duell(self, ctx, user:User, width:int=7, height:int=6):
+        msg = await ctx.sendEmbed(
+            title="Vier Gewinnt",
+            color=0x0078D7,
+            description=f"Duell gegen {user.mention} wird erstellt..."
+        )
+
+        game = await ctx.database._createVierGewinntGame(channel_id=str(ctx.channel.id), message_id=str(msg.id), player_1_id=str(ctx.author.id), player_2_id=str(user.id), width=width, height=height)
+
+        embed = ctx.bot.getEmbed(
+            title=f"Vier Gewinnt (#{game.pk})",
+            color=0x0078D7,
+            description=game.get_description()
+        )
+
+        await msg.edit(embed=embed)
+
+        for emoji in VIERGEWINNT_NUMBER_EMOJIS[:game.width]:
+            await msg.add_reaction(emoji)
+
 def setup(bot):
     bot.add_cog(Games(bot))
